@@ -1,7 +1,16 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
 /* scsi primary commands */
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include <unistd.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -27,6 +36,14 @@
 #include "libdax_msgs.h"
 extern struct libdax_msgs *libdax_messenger;
 
+/* ts A91111 :
+   whether to log SCSI commands:
+   bit0= log in /tmp/libburn_sg_command_log
+   bit1= log to stderr
+   bit2= flush every line
+*/
+extern int burn_sg_log_scsi;
+
 
 /* spc command set */
 /* ts A70519 : allocation length byte 3+4 was 0,255 */
@@ -36,8 +53,7 @@ static unsigned char SPC_INQUIRY[] = { 0x12, 0, 0, 0, 36, 0 };
 static unsigned char SPC_PREVENT[] = { 0x1e, 0, 0, 0, 1, 0 };
 static unsigned char SPC_ALLOW[] = { 0x1e, 0, 0, 0, 0, 0 };
 static unsigned char SPC_MODE_SENSE[] = { 0x5a, 0, 0, 0, 0, 0, 0, 16, 0, 0 };
-static unsigned char SPC_MODE_SELECT[] =
-	{ 0x55, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+static unsigned char SPC_MODE_SELECT[] = { 0x55, 16, 0, 0, 0, 0, 0, 0, 0, 0 };
 static unsigned char SPC_REQUEST_SENSE[] = { 0x03, 0, 0, 0, 18, 0 };
 static unsigned char SPC_TEST_UNIT_READY[] = { 0x00, 0, 0, 0, 0, 0 };
 
@@ -100,7 +116,8 @@ int spc_test_unit_ready(struct burn_drive *d)
 int spc_wait_unit_attention(struct burn_drive *d, int max_sec, char *cmd_text,
 				int flag)
 {
-	int i, ret = 1, key = 0, asc = 0, ascq = 0;
+	int i, ret = 1, key = 0, asc = 0, ascq = 0, clueless_start = 0;
+	static int clueless_timeout = 5 * 10;
 	char msg[320];
 	unsigned char sense[14];
 	enum response resp;
@@ -130,6 +147,7 @@ int spc_wait_unit_attention(struct burn_drive *d, int max_sec, char *cmd_text,
 				/* media change notice = try again */
 				goto slumber;
 
+handle_error:;
 			/* ts A90213 */
 			sprintf(msg,
 				"Asynchronous SCSI error on %s: ", cmd_text);
@@ -144,7 +162,23 @@ int spc_wait_unit_attention(struct burn_drive *d, int max_sec, char *cmd_text,
 				msg, 0, 0);
 			d->cancel = 1;
 	break;
-		}
+		} else if (ascq == 0x00) { /* CAUSE NOT REPORTABLE */
+			/* Might be a clueless system adapter */
+			if (clueless_start == 0)
+				clueless_start = i;
+			if (i - clueless_start > clueless_timeout) {
+				libdax_msgs_submit(libdax_messenger,
+				  d->global_index,
+				  0x00000002,
+				  LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+				  "Ended clueless NOT READY cycle",
+				  0, 0);
+				ret = 1; /* medium not present = ok */
+	break;
+			}
+		} else if (ascq == 0x02 || ascq == 0x03) 
+			goto handle_error;
+
 slumber:;
 		usleep(100000);
 	}
@@ -321,7 +355,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 		was_error = 1;
 	}
 
-	size = c.page->data[0] * 256 + c.page->data[1];
+	size = c.page->data[0] * 256 + c.page->data[1] + 2;
 	page = c.page->data + 8;
 
 	/* ts A61225 :
@@ -517,7 +551,7 @@ void spc_sense_error_params(struct burn_drive *d)
 	c.dir = FROM_DRIVE;
 	d->issue_command(d, &c);
 
-	size = c.page->data[0] * 256 + c.page->data[1];
+	size = c.page->data[0] * 256 + c.page->data[1] + 2;
 	m = d->mdata;
 	page = c.page->data + 8;
 	d->params.retries = page[3];
@@ -600,7 +634,7 @@ void spc_sense_write_params(struct burn_drive *d)
 	/* ts A71128 : do not interpret reply if error */
 	m = d->mdata;
 	if (!c.error) {
-		size = c.page->data[0] * 256 + c.page->data[1];
+		size = c.page->data[0] * 256 + c.page->data[1] + 2;
 		page = c.page->data + 8;
 		burn_print(1, "write page length 0x%x\n", page[1]);
 		m->write_page_length = page[1];
@@ -925,18 +959,36 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 			     int *key, int *asc, int *ascq)
 {
 	char *msg;
+	static char key_def[16][40] = {
+		"(no specific error)",
+		"Recovered error",
+		"Drive not ready",
+		"Medium error",
+		"Drive error",
+		"Illegal request",
+		"Drive event",
+		"Data protected",
+		"Blank/Nonblank",
+		"Vendor specific code",
+		"Copy aborted",
+		"Command aborted",
+		"(obsolete error code)",
+		"Volume overflow",
+		"Miscompare",
+		"(reserved error code)",
+	};
 
 	msg= msg_data;
 	*key= *asc= *ascq= -1;
 
 	if (senselen<=0 || senselen>2)
-		*key = sense[2];
+		*key = sense[2] & 0x0f;
 	if (senselen<=0 || senselen>12)
 		*asc = sense[12];
 	if (senselen<=0 || senselen>13)
 		*ascq = sense[13];
 
-	sprintf(msg, "[%X %2.2X %2.2X]  ", *key, *asc, *ascq);
+	sprintf(msg, "[%X %2.2X %2.2X] ", *key, *asc, *ascq);
 	msg= msg + strlen(msg);
 
 	burn_print(12, "CONDITION: 0x%x 0x%x 0x%x on %s %s\n",
@@ -951,8 +1003,11 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 		sprintf(msg, "Not ready");
 		return RETRY;
 	case 0x04:
-		sprintf(msg,
+		if (*ascq == 1)
+			sprintf(msg,
 			"Logical unit is in the process of becoming ready");
+		else
+			sprintf(msg, "Logical unit is not ready");
 		return RETRY;
 	case 0x08:
 		if (*key != 4)
@@ -1205,8 +1260,8 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 		return FAIL;
 	}
 	sprintf(msg_data,
-		"Failure. See mmc3r10g.pdf: Sense Key %X ASC %2.2X ASCQ %2.2X",
-		*key, *asc, *ascq);
+		"See MMC specs: Sense Key %X \"%s\", ASC %2.2X ASCQ %2.2X",
+		*key & 0xf, key_def[(*key) & 0xf], *asc, *ascq);
 	return FAIL;
 }
 
@@ -1412,3 +1467,56 @@ int scsi_show_cmd_reply(struct command *c, void *fp_in, int flag)
 		fprintf(fp, "\n");
 	return 1;
 }
+
+
+/* ts A91218 (former sg_log_cmd ts A70518) */ 
+/** Logs command (before execution) */
+int scsi_log_cmd(struct command *c, void *fp_in, int flag)
+{
+	FILE *fp = fp_in;
+
+	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
+		scsi_show_cmd_text(c, fp, 0);
+		if (burn_sg_log_scsi & 4)
+			fflush(fp);
+	}
+	if (fp == stderr || !(burn_sg_log_scsi & 2))
+		return 1;
+	scsi_log_cmd(c, stderr, flag);
+	return 1;
+}
+
+
+/* ts A91221 (former sg_log_err ts A91108) */
+/** Logs outcome of a sg command.
+    @param flag  bit0 causes an error message
+                 bit1 do not print duration
+*/
+int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
+		 int duration, int flag)
+{
+	char durtxt[20];
+	FILE *fp = fp_in;
+
+      	if(fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
+		if(flag & 1) {
+			durtxt[0] = 0;
+			if (!(flag & 2))
+  				sprintf(durtxt, "   (%6d ms)\n",duration);
+  			fprintf(fp, "+++ key=%X  asc=%2.2Xh  ascq=%2.2Xh%s\n",
+				sense[2], sense[12], sense[13], durtxt);
+
+		} else {
+			scsi_show_cmd_reply(c, fp, 0);
+			if (!(flag & 2))
+				fprintf(fp,"%6d ms\n", duration);
+		}
+		if (burn_sg_log_scsi & 4)
+			fflush(fp);
+	}
+	if (fp == stderr || !(burn_sg_log_scsi & 2))
+		return 1;
+	scsi_log_err(c, stderr, sense, duration, flag);
+	return 1;
+}
+
